@@ -16,6 +16,7 @@ import types
 import shutil
 import numbers
 import traceback
+import fcntl
 
 # i still like this
 from pydoc import locate
@@ -74,7 +75,7 @@ def get_chunk_store(path):
     try:
         chunk_store = ZARR_CHUNK_STORES[path]
     except KeyError:
-        chunk_store = zarr.DirectoryStore(path)
+        chunk_store = zarr.storage.LocalStore(path)
         ZARR_CHUNK_STORES[path] = chunk_store
 
     return chunk_store
@@ -196,7 +197,9 @@ class Serializer:
 
         attrs = None
 
-        if hasattr(obj, "__savestate__"):
+        if isinstance(obj, dict):
+            attrs = obj
+        elif hasattr(obj, "__savestate__"):
             attrs = obj.__savestate__()
         elif hasattr(obj, "__getstate__"):
             attrs = obj.__getstate__()
@@ -204,8 +207,6 @@ class Serializer:
             attrs = obj.__dict__
         elif hasattr(obj, "__slots__"):
             attrs = {n: getattr(obj, n) for n in obj.__slots__}
-        elif isinstance(obj, dict):
-            attrs = obj
 
         if attrs is None:
             # in case we don't find any serializeable attributes
@@ -295,21 +296,31 @@ class Loader:
             # handles general objects and dicts
 
             if hasattr(obj, "__loadstate__"):
-                ld = {k: self.load(None, v) for k, v in json_obj.items()}
+                ld = {}
+                for k, v in json_obj.items():
+                    nv = self.load(None, v)
+                    if nv is not Skip:
+                        ld[k] = nv
                 if "__class__" in ld:
                     del ld["__class__"]
                 obj.__loadstate__(ld)
             elif isinstance(obj, dict) and len(obj) == 0:
                 # if obj is dict-like and empty,
                 # we accept whatever is stored in the json file
-                ld = {k: self.load(None, v) for k, v in json_obj.items()}
+                ld = {}
+                for k, v in json_obj.items():
+                    nv = self.load(None, v)
+                    if nv is not Skip:
+                        ld[k] = nv
                 if "__class__" in ld:
                     del ld["__class__"]
                 obj.update(ld)
             else:
                 for k, v in attrs.items():
                     if k in json_obj:
-                        ext_setattr(obj, k, self.load(v, json_obj[k]))
+                        nv = self.load(v, json_obj[k])
+                        if nv is not Skip:
+                            ext_setattr(obj, k, nv)
             return obj
         elif (t == list or t == tuple) and jt == list:
             # handles lists and tuples
@@ -342,7 +353,8 @@ class Loader:
                     print(f"Warning: skipping init of unknown type {cls_name}")
                     return Skip
             elif jt == list:
-                return [self.load(None, v) for v in json_obj]
+                res = [self.load(None, v) for v in json_obj]
+                return [v for v in res if v is not Skip]
             else:
                 # just use whatever is contained in json
                 return json_obj
@@ -373,28 +385,41 @@ def save(obj,
     The directory will be created if it not already exists.
     """
 
-    os.makedirs(directory, exist_ok=True)
-
-    ser = Serializer(directory,
-                     hide_private=hide_private,
-                     compressor=compressor,
-                     mmap_arrays=mmap_arrays,
-                     externalize=externalize)
-    rep = ser.serialize(obj)
-
-    if rep is Skip:
-        return False
-
-    unfinished_path = os.path.join(directory, "unfinished.json")
-
-    with open(unfinished_path, "w+") as fd:
-        json.dump(rep, fd, indent=2)
-
     state_path = os.path.join(directory, "state.json")
 
-    os.rename(unfinished_path, state_path)
+    try:
+        state_fd = open(state_path, "r+")
+    except FileNotFoundError:
+        state_fd = None
 
-    clean_zarr_store(ser)
+    try:
+        if state_fd is not None:
+            fcntl.flock(state_fd, fcntl.LOCK_EX)
+
+        os.makedirs(directory, exist_ok=True)
+
+        ser = Serializer(directory,
+                         hide_private=hide_private,
+                         compressor=compressor,
+                         mmap_arrays=mmap_arrays,
+                         externalize=externalize)
+        rep = ser.serialize(obj)
+
+        if rep is Skip:
+            return False
+
+        unfinished_path = os.path.join(directory, "unfinished.json")
+
+        with open(unfinished_path, "w+") as fd:
+            json.dump(rep, fd, indent=2)
+
+        clean_zarr_store(ser)
+
+        os.rename(unfinished_path, state_path)
+    finally:
+        if state_fd is not None:
+            fcntl.flock(state_fd, fcntl.LOCK_UN)
+            state_fd.close()
 
     return True
 
@@ -417,14 +442,21 @@ def load(obj, path, mmap_arrays=False):
 
     state_path = os.path.join(path, "state.json")
 
-    if not os.path.exists(state_path):
+    try:
+        state_fd = open(state_path, "r+")
+    except FileNotFoundError:
         return False
 
-    with open(state_path, "r") as fd:
-        json_state = json.load(fd)
+    try:
+        fcntl.flock(state_fd, fcntl.LOCK_EX)
 
-    lod = Loader(path, mmap_arrays)
-    lod.load(obj, json_state)
+        json_state = json.load(state_fd)
+
+        lod = Loader(path, mmap_arrays)
+        lod.load(obj, json_state)
+    finally:
+        fcntl.flock(state_fd, fcntl.LOCK_UN)
+        state_fd.close()
 
     return True
 
